//
// chipset.c - the feature / chipset class definitions
//   by Ed Porras <ep@bangchemistry.com>
//   with portions of code from
//
//     wmgtemp by Roger Dunce <kronos@nanunanu.org> 
//
//
// $Log: chipset.c,v $
// Revision 1.1  2003/03/25 02:14:20  ep
// Initial revision
//
//

#include <iostream>
#include <iomanip>
#include <string>
#include <cstring>
#include <vector>
#include <algorithm>
#include <sensors/sensors.h>

#include "chipset_mgr.h"
#include "chipset.h"

using namespace std;

namespace wmmsens
{
  // predicate function object used by the STL algorithm.. returns true
  // if the feature has an empty mapped_feature list
  struct hasMappedFeat {
    bool operator()(cs_feature *f) { 
      if (f->numMappings() != 0) 
	return false;
      delete f;
      return true;
    }
  };

  // =========================================================================
  // the chipset class

  //
  // constructor
  chipset::chipset(const sensors_chip_name& name, ui16 bits) : 
    chipname(name), poll_bits(bits)
  {
    // deep copy the name
    chipname.prefix = new char[ strlen(name.prefix) + 1 ];
    strcpy( chipname.prefix, name.prefix );
  }

  //
  // destructor
  chipset::~chipset() 
  {
    for (ui32 i = 0; i < feature_list.size(); i++)
      delete feature_list[i];
    delete [] chipname.prefix;
  }

  //
  // gets the features of this chipset
  bool chipset::getFeatures(ui8 history_steps)
  {
    bool save = false;
    int nr1 = 0, nr2 = 0;

    // one by one..
    while (true)
    {
      const sensors_feature_data* sfd = sensors_get_all_features(chipname, 
								 &nr1, 
								 &nr2);
      // didn't find another one
      if (sfd == NULL)
      {
	// if we are keeing this one, purge those with nothing mapped
	// to them
	if (save)
	  feature_list.erase( remove_if( feature_list.begin(), 
					 feature_list.end(), 
					 hasMappedFeat() ), 
			      feature_list.end() );
	break;
      }

      // check if we should ignore this feature
      if (sensors_get_ignored(chipname, sfd->number) <= 0)
	continue;

      if (chipset_mgr::verbose)
	cout << "  +-- got feature id: " << setw(2) << sfd->number 
	     << ", mapping: " << setw(2) << sfd->mapping 
	     << ", name: \"" << sfd->name << "\"" << endl;

      // check if it is not mapped to another
      if (sfd->mapping == -1)
      {
	// get this feature's info
	std::string name = sfd->name;
	cs_feature::csf_type type = cs_feature::UNKNOWN;
	ui8 step_cnt = 1;

	// if we find something with the word "temp", we want to keep
	// this chipset.. kinda primitive, I know.. but..
	if (name.find("temp") != string::npos ||
	    name.find("Temp") != string::npos)
	{
	  if (!(poll_bits & chipset::POLL_TEMP))
	    continue;

	  type = cs_feature::TEMPERATURE;
	  step_cnt = history_steps;
	  save = true;
	}
	else if (name.find("fan") != string::npos)
	{
	  if (!(poll_bits & chipset::POLL_FAN))
	    continue;

	  type = cs_feature::FAN;
	}
	else 
	{
	  // assume voltage for now since there is no specific
	  // keyword we can look for..  features with no limits will
	  // be discarded later..  !!check into this with other
	  // chipsets!!
	  if (!(poll_bits & chipset::POLL_VOLTAGE))
	    continue;

	  type = cs_feature::VOLTAGE;
	}

	// create a new instance and add it to the bunch
	cs_feature *f = new cs_feature( chipname, *sfd, type, step_cnt );
	feature_list.push_back( f );
      }
      else
      {
	// the feature is mapped to another one so we don't store it
	// on the main list.. if the main type was not stored, this
	// is discarded
	storeMappedFeature( *sfd );
      }
    }

    return save;
  }


  //
  // finds the feature in the list that the new one is mapped to and
  // adds it to its list of mapped features
  bool chipset::storeMappedFeature(const sensors_feature_data& sfd)
  {
    for (register ui32 i = 0; i < feature_list.size(); i++)
      if (feature_list[i]->id() == sfd.mapping)
      {
	cs_feature *f = new cs_feature( chipname, sfd, 
					feature_list[i]->type() );

	bool rslt = feature_list[i]->addMappedFeature( *f );

	if (!rslt)
	  delete f;
	return rslt;
      }
    return false;
  }

  //
  // forces updates of chipset features.. if all is true, then mapped
  // features are updated as well
  void chipset::update(bool all) 
  {
    for (register ui32 i = 0; i < feature_list.size(); i++)
      feature_list[ i ]->update( all );
  }


  //
  // output op
  ostream& operator<<(ostream &o, const chipset& cs)
  {
    o << "  Chipset: " << cs.chipname.prefix << " - " << 
      cs.feature_list.size() << " feature(s) registered" << endl;

    for (register ui32 i = 0; i < cs.feature_list.size(); i++)
      o << "  " << (i + 1) << ". " << *cs.feature_list[i] << endl;
    return o;
  }

} // namespace wmmsens
