#ifndef WMMSENS_CS_FEATURE_H
#define WMMSENS_CS_FEATURE_H

//
// cs_feature.h - new implementation of sensor manager classes
//   by Ed Porras <ep@bangchemistry.com>
//   with portions of code from
//
//     wmgtemp by Roger Dunce <kronos@nanunanu.org> 
//
//
// $Log: cs_feature.h,v $
// Revision 1.1  2003/03/25 02:14:20  ep
// Initial revision
//
// 

#include <iosfwd>
#include <string>
#include <vector>

#include <stdint.h>
#include <sensors/sensors.h>

//
// handy typedefs
typedef u_int8_t  ui8;
typedef u_int16_t ui16;
typedef u_int32_t ui32;

namespace wmmsens
{
  // define this if you want to maintain doubles for history values
#ifdef DBL_HIST_VAL
  typedef double featval;
#else
  typedef float featval;
#endif

  //
  // feature information
  class cs_feature
  {
  public:
    // the types of features we look for
    enum csf_type { 
      TEMPERATURE,
      FAN,
      VOLTAGE,
      UNKNOWN,  // undefined type
    };

  private:
    // reference to struct stored in chipset
    const sensors_chip_name& chipname;

    // copy of the struct obtained from lm_sensors
    sensors_feature_data f_data;

    // the feature's label
    std::string lbl;

    // stores the value(s) read
    std::vector<featval> val;

    // store the min & max of the current values maintained
    featval min_read, max_read;

    // feature type.. 
    csf_type f_type;

    // list of features that are mapped to this one
    std::vector<cs_feature *> mapped_feature;

    // index of the min / max features that are mapped to this one
    int min_idx, max_idx;

    // set true if value is not within proper limits
    bool alarm_on;

    // prohibit
    cs_feature(const cs_feature&);
    cs_feature& operator=(const cs_feature&);

  public:
    ~cs_feature(); 

    // getters
    int           id() const      { return f_data.number; }
    std::string   name() const    { return f_data.name; }
    const std::string& label() const   { return lbl; }
    int           mapping() const { return f_data.mapping; }
    csf_type      type() const    { return f_type; }
    bool          alarm() const   { return alarm_on; }

    featval value() const { return val.back(); } // latest value read
    featval value(int idx) const { return val[idx]; } 
    ui8 numSteps() const { return val.size(); }
    ui32 numMappings() const { return mapped_feature.size(); }

    featval min() const { return min_read; }
    featval max() const { return max_read; }

    bool history() const { return (val.size() > 1); }

    // output 
    friend std::ostream& operator<<(std::ostream &, const cs_feature&);

  private:
    friend class chipset;

    // constructor & destructor
    cs_feature(const sensors_chip_name& name,
	       const sensors_feature_data& fd, 
	       csf_type type, ui8 steps = 1);

    // adds a feature mapped to this one
    bool addMappedFeature(cs_feature& f);

    // reads latest data from sensor lib
    void update(bool all);
  };

} // namespace wmmsens

#endif
